/*!
 * @file rtc_main.h
 * @brief RTUnit component base functions
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2005 Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 */
/*
 * RTC-PIC command format
 * |:byte|
 *
 * Input command: proxy->RTUnit
 * |Header:1|Top Level Command:1|Command data:0-N|Parity:1|
 *
 * Top level command
 * RTC_CMD:  RT-Component command
 * INPORT:   InPort command
 * OUTPORT:  OutPort command
 * USER_CMD: User defined command
 *
 * Return value:
 * |Header:1|Top Level Command:1|Command data|RTM_OK or ERR:1|Parity:1|
 *
 * Output command: RTUnit->proxy
 * |Header:1|Top Level Command:1|Command data:0-N|Parity:1|
 *
 */

#ifndef RTC_BASE_H
#define RTC_BASE_H

// RTM OK/ERR
#define RTM_OK           0x00
#define RTM_ERR          0x01
#define RTM_FATAL_ERR    0x02
#define RTM_PARITY_ERR   0x03

// OK return / error return
#define RTM_ACK_OK          rtm_ack(RTM_OK);
#define RTM_ACK_ERR         rtm_ack(RTM_ERR);
#define RTM_ACK_FATAL_ERR   rtm_ack(RTM_FATAL_ERR);
#define RTM_ACK_PARITY_ERR  rtm_ack(RTM_PARITY_ERR);

// RTComponent state
#define RTC_UNKNOWN      0x00
#define RTC_CREATED      0x01 //RTC_BORN
#define RTC_STOPPED		 0x02
#define RTC_RUNNING		 0x03
#define RTC_INACTIVE     0x04 //READY
#define RTC_ACTIVE		 0x05
#define RTC_ERROR		 0x06
#define RTC_RESETTING	 0x07


//#define RTC_STARTING     0x04
//#define RTC_ACTIVE       0x05
//#define RTC_STOPPING     0x06
//#define RTC_ABORTING     0x07
//#define RTC_ERROR        0x08
//#define RTC_FATAL_ERROR  0x09
//#define RTC_EXITING      0x0A


//============================================================
// Command definition
//============================================================
// Top level command
#define RTC_PROFILE      0x00
#define RTC_CMD          0x01
#define INPORT_CMD       0x02
#define OUTPORT_CMD      0x03
#define RTC_ACK			 0x01
// user define command
#define USER_CMD         0x10
#define RTUNIT_CMD       0xF0

// RTC_CMD sub command
#define RTC_START        0x00
#define RTC_STOP         0x01
#define RTC_RESET        0x02
#define RTC_ERR_TEST	 0x03
//#define RTC_EXIT         0x03
//#define RTC_KILL         0x04
#define RTC_STATE        0x10
#define RTC_INPORTS      0x11
#define RTC_OUTPORTS     0x12
//==========================================================
//#define RTC_OK		   	 0x20
//#define RTC_ERR			 0x21
//=========================================================
//============================================================
// Message from RS232C handling functions
//============================================================
#define CMD_LEVEL0 1
#define CMD_LEVEL1 2
#define CMD_LEVEL2 3
#define CMD_LEVEL3 4
#define RS_TIMEOUT 80

// RTUNIT_CMD sub command
#define RTUNIT_RESET     0x00   // ϡɥEå
#define RTUNIT_SLEEP     0x01   // ꡼ץ⡼
#define RTUNIT_SET_IP    0x02   // XPort μIPɥE򥻥å
#define RTUNIT_SET_SVR   0x03  


typedef BYTE RtmRes;
typedef BYTE ComponentState;

ComponentState g_CurrState = RTC_UNKNOWN;
ComponentState g_NextState = RTC_CREATED;


//============================================================
// Prototype declaration
//============================================================
void rtc_mainloop();
RtmRes _check_error(RtmRes result);
void rtc_connect_proxy();
void rtc_parse_cmd();

// return func
void rtm_ack(BYTE ret);

// RTC commands
void _rtc_profile();
void _rtc_init();
void _rtc_state();
void _rtc_inports();
void _rtc_outports();
void _rtc_shutdown();

// State transition
RtmRes _rtc_start();
RtmRes _rtc_stop();
RtmRes _rtc_reset();
RtmRes _rtc_error();
//RtmRes _rtc_exit();
//RtmRes _rtc_kill();

// Activity
RtmRes _rtc_state_do(ComponentState state);
RtmRes _rtc_state_entry(ComponentState state);
RtmRes _rtc_state_exit(ComponentState state);


// Activity implementation
RtmRes rtc_on_initialize();
RtmRes rtc_on_startup();
RtmRes rtc_on_shutdown();
RtmRes rtc_on_activate();
RtmRes rtc_on_aborting();
RtmRes rtc_on_error();
RtmRes rtc_on_reset();
RtmRes rtc_on_deactivate();
RtmRes rtc_on_execute();


#endif // RTC_BASE_H